@@ -23,6 +23,7 @@
 #include "gtd-plugin-manager.h"
 #include "gtd-task.h"
 #include "gtd-task-list.h"
+#include "gtd-timer.h"
 
 #include <glib/gi18n.h>
 
@@ -50,6 +51,7 @@ typedef struct
   GList                 *providers;
   GList                 *panels;
   GtdProvider           *default_provider;
+  GtdTimer              *timer;
 } GtdManagerPrivate;
 
 struct _GtdManager
@@ -82,6 +84,8 @@ enum
 {
   PROP_0,
   PROP_DEFAULT_PROVIDER,
+  PROP_DEFAULT_TASKLIST,
+  PROP_TIMER,
   PROP_PLUGIN_MANAGER,
   LAST_PROP
 };
@@ -99,25 +103,25 @@ check_provider_is_default (GtdManager  *manager,
   default_provider = g_settings_get_string (priv->settings, "default-provider");
 
   if (g_strcmp0 (default_provider, gtd_provider_get_id (provider)) == 0)
-    {
-      g_set_object (&priv->default_provider, provider);
-
-      g_object_notify (G_OBJECT (manager), "default-provider");
-    }
+    gtd_manager_set_default_provider (manager, provider);
 
   g_free (default_provider);
 }
 
 static void
-emit_show_error_message (GtdManager  *manager,
-                         const gchar *primary_text,
-                         const gchar *secondary_text)
+emit_show_error_message (GtdManager                *manager,
+                         const gchar               *primary_text,
+                         const gchar               *secondary_text,
+                         GtdNotificationActionFunc  action,
+                         gpointer                   user_data)
 {
   g_signal_emit (manager,
                  signals[SHOW_ERROR_MESSAGE],
                  0,
                  primary_text,
-                 secondary_text);
+                 secondary_text,
+                 action,
+                 user_data);
 }
 
 static void
@@ -127,6 +131,7 @@ gtd_manager_finalize (GObject *object)
 
   g_clear_object (&self->priv->plugin_manager);
   g_clear_object (&self->priv->settings);
+  g_clear_object (&self->priv->timer);
 
   G_OBJECT_CLASS (gtd_manager_parent_class)->finalize (object);
 }
@@ -145,6 +150,14 @@ gtd_manager_get_property (GObject    *object,
       g_value_set_object (value, priv->default_provider);
       break;
 
+    case PROP_DEFAULT_TASKLIST:
+      g_value_set_object (value, gtd_provider_get_default_task_list (priv->default_provider));
+      break;
+
+    case PROP_TIMER:
+      g_value_set_object (value, priv->timer);
+      break;
+
     case PROP_PLUGIN_MANAGER:
       g_value_set_object (value, priv->plugin_manager);
       break;
@@ -160,7 +173,11 @@ gtd_manager_set_property (GObject      *object,
                           const GValue *value,
                           GParamSpec   *pspec)
 {
-  GtdManagerPrivate *priv = gtd_manager_get_instance_private (GTD_MANAGER (object));
+  GtdManagerPrivate *priv;
+  GtdManager *self;
+
+  self = GTD_MANAGER (object);
+  priv = gtd_manager_get_instance_private (self);
 
   switch (prop_id)
     {
@@ -169,6 +186,10 @@ gtd_manager_set_property (GObject      *object,
         g_object_notify (object, "default-provider");
       break;
 
+    case PROP_DEFAULT_TASKLIST:
+      gtd_manager_set_default_task_list (self, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -197,6 +218,34 @@ gtd_manager_class_init (GtdManagerClass *klass)
                              GTD_TYPE_PROVIDER,
                              G_PARAM_READWRITE));
 
+  /**
+   * GtdManager::default-task-list:
+   *
+   * The default provider.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_DEFAULT_TASKLIST,
+        g_param_spec_object ("default-task-list",
+                             "The default task list of the application",
+                             "The default task list of the application",
+                             GTD_TYPE_TASK_LIST,
+                             G_PARAM_READWRITE));
+
+  /**
+   * GtdManager::timer:
+   *
+   * The underlying timer of GNOME To DO.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_TIMER,
+        g_param_spec_object ("timer",
+                             "The timer",
+                             "The timer of the application",
+                             GTD_TYPE_TIMER,
+                             G_PARAM_READABLE));
+
   /**
    * GtdManager::plugin-manager:
    *
@@ -272,7 +321,9 @@ gtd_manager_class_init (GtdManagerClass *klass)
    * GtdManager::show-error-message:
    * @manager: a #GtdManager
    * @primary_text: the primary message
-   * @secondary_text: the detailed explanation of the error
+   * @secondary_text: the detailed explanation of the error or the text to the notification button.
+   * @action : optionally action of type GtdNotificationActionFunc ignored if it's null.
+   * @user_data : user data passed to the action.
    *
    * Notifies about errors, and sends the error message for widgets
    * to display.
@@ -285,9 +336,11 @@ gtd_manager_class_init (GtdManagerClass *klass)
                                               NULL,
                                               NULL,
                                               G_TYPE_NONE,
-                                              2,
+                                              4,
                                               G_TYPE_STRING,
-                                              G_TYPE_STRING);
+                                              G_TYPE_STRING,
+                                              G_TYPE_POINTER,
+                                              G_TYPE_POINTER);
 
   /**
    * GtdManager::panel-added:
@@ -366,6 +419,14 @@ gtd_manager_class_init (GtdManagerClass *klass)
                                             GTD_TYPE_PROVIDER);
 }
 
+static void
+gtd_manager__default_list_changed_cb (GtdProvider *provider,
+                                      GParamSpec  *pspec,
+                                      GtdManager  *self)
+{
+  g_object_notify (G_OBJECT (self), "default-task-list");
+}
+
 static void
 gtd_manager__task_list_modified (GtdTaskList *list,
                                  GtdTask     *task,
@@ -507,14 +568,34 @@ gtd_manager__provider_removed (GtdPluginManager *plugin_manager,
   for (l = lists; l != NULL; l = l->next)
     gtd_manager__list_removed (provider, l->data, self);
 
+  /* Disconnect old signals */
+  
+  g_signal_handlers_disconnect_by_func (provider,
+                                        gtd_manager__default_list_changed_cb,
+                                        self);
+
+  g_signal_handlers_disconnect_by_func (provider,
+                                        gtd_manager__list_added,
+                                        self);
+
+  g_signal_handlers_disconnect_by_func (provider,
+                                        gtd_manager__list_changed,
+                                        self);
+
+  g_signal_handlers_disconnect_by_func (provider,
+                                        gtd_manager__list_removed,
+                                        self);
+
   g_signal_emit (self, signals[PROVIDER_REMOVED], 0, provider);
 }
+
 static void
 gtd_manager_init (GtdManager *self)
 {
   self->priv = gtd_manager_get_instance_private (self);
   self->priv->settings = g_settings_new ("org.gnome.todo");
   self->priv->plugin_manager = gtd_plugin_manager_new ();
+  self->priv->timer = gtd_timer_new ();
 }
 
 /**
@@ -604,6 +685,11 @@ gtd_manager_update_task (GtdManager *manager,
   g_return_if_fail (GTD_IS_TASK (task));
 
   list = gtd_task_get_list (task);
+
+  /* Task does not have parent list, nothing we can do */
+  if (!list)
+      return;
+
   provider = gtd_task_list_get_provider (list);
 
   gtd_provider_update_task (provider, task);
@@ -744,21 +830,23 @@ gtd_manager_get_default_provider (GtdManager *manager)
 }
 
 /**
- * gtd_manager_set_default_storage:
+ * gtd_manager_set_default_provider:
  * @manager: a #GtdManager
- * @default_storage: (nullable): the default storage location.
+ * @provider: (nullable): the default provider.
  *
- * Sets the default storage location id.
+ * Sets the provider.
  */
 void
 gtd_manager_set_default_provider (GtdManager  *manager,
                                   GtdProvider *provider)
 {
   GtdManagerPrivate *priv;
+  GtdProvider *previous;
 
   g_return_if_fail (GTD_IS_MANAGER (manager));
 
   priv = manager->priv;
+  previous = priv->default_provider;
 
   if (g_set_object (&priv->default_provider, provider))
     {
@@ -766,10 +854,78 @@ gtd_manager_set_default_provider (GtdManager  *manager,
                              "default-provider",
                              provider ? gtd_provider_get_id (provider) : "local");
 
+      /* Disconnect the previous provider... */
+      if (previous)
+        {
+          g_signal_handlers_disconnect_by_func (previous,
+                                                gtd_manager__default_list_changed_cb,
+                                                manager);
+        }
+
+      /* ... and connect the current one */
+      if (provider)
+        {
+          g_signal_connect (provider,
+                            "notify::default-task-list",
+                            G_CALLBACK (gtd_manager__default_list_changed_cb),
+                            manager);
+        }
+
       g_object_notify (G_OBJECT (manager), "default-provider");
+      g_object_notify (G_OBJECT (manager), "default-task-list");
     }
 }
 
+/**
+ * gtd_manager_get_default_task_list:
+ * @self: a #GtdManager
+ *
+ * Retrieves the default tasklist of the default provider.
+ *
+ * Returns: (transfer none)(nullable): a #GtdTaskList
+ */
+GtdTaskList*
+gtd_manager_get_default_task_list (GtdManager *self)
+{
+  GtdManagerPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_MANAGER (self), NULL);
+
+  priv = gtd_manager_get_instance_private (self);
+
+  if (!priv->default_provider)
+    return NULL;
+
+  return gtd_provider_get_default_task_list (priv->default_provider);
+}
+
+/**
+ * gtd_manager_set_default_task_list:
+ * @self: a #GtdManager
+ * @list: (nullable): a #GtdTaskList, or %NULL
+ *
+ * Sets the default task list of the application.
+ */
+void
+gtd_manager_set_default_task_list (GtdManager  *self,
+                                   GtdTaskList *list)
+{
+  g_return_if_fail (GTD_IS_MANAGER (self));
+  g_return_if_fail (GTD_IS_TASK_LIST (list));
+
+  if (list)
+    {
+      GtdProvider *provider;
+
+      provider = gtd_task_list_get_provider (list);
+
+      gtd_manager_set_default_provider (self, provider);
+      gtd_provider_set_default_task_list (provider, list);
+    }
+
+  g_object_notify (G_OBJECT (self), "default-task-list");
+}
+
 /**
  * gtd_manager_get_settings:
  * @manager: a #GtdManager
@@ -821,15 +977,40 @@ gtd_manager_set_is_first_run (GtdManager *manager,
 }
 
 void
-gtd_manager_emit_error_message (GtdManager  *manager,
-                                const gchar *primary_message,
-                                const gchar *secondary_message)
+gtd_manager_emit_error_message (GtdManager                *manager,
+                                const gchar               *primary_message,
+                                const gchar               *secondary_message,
+                                GtdNotificationActionFunc  function,
+                                gpointer                   user_data)
 {
   g_return_if_fail (GTD_IS_MANAGER (manager));
 
   emit_show_error_message (manager,
                            primary_message,
-                           secondary_message);
+                           secondary_message,
+                           function,
+                           user_data);
+}
+
+/**
+ * gtd_manager_get_timer:
+ * @self: a #GtdManager
+ *
+ * Retrieves the #GtdTimer from @self. You can use the
+ * timer to know when your code should be updated.
+ *
+ * Returns: (transfer none): a #GtdTimer
+ */
+GtdTimer*
+gtd_manager_get_timer (GtdManager *self)
+{
+  GtdManagerPrivate *priv;
+
+  g_return_val_if_fail (GTD_IS_MANAGER (self), NULL);
+
+  priv = gtd_manager_get_instance_private (self);
+
+  return priv->timer;
 }
 
 void
